/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
-------------------------- END-OF-HEADER -----------------------------

File    : BSP_UART_Template.c
Purpose : Template for BSP UART implementation
*/

#include "BSP_UART.h"
#include "RTOS.h"     // For OS_Enter/LeaveInterrupt() and OS_GetPClkTimer() .

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
// None

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/
// None

/*********************************************************************
*
*       Types, local
*
**********************************************************************
*/
// None

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
// None

/*********************************************************************
*
*       Prototypes
*
*  Declare ISR handler here to avoid "no prototype" warning.
*  They are not declared in any CMSIS header.
*
**********************************************************************
*/

#ifdef __cplusplus
extern "C" {
#endif

void UART_RX_TX_IRQHandler(void);
void UART_ERR_IRQHandler(void);

#ifdef __cplusplus
}
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/
// None


/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       UART_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are now in interrupt context.
*/
void UART_RX_TX_IRQHandler(void) {
  OS_EnterInterrupt();
  // Add code here.
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       UART_ERR_IRQHandler()
*
*  Function description
*    UART error interrupt handler.
*
*  Additional information
*    Needs to inform the OS that we are now in interrupt context.
*/
void UART_ERR_IRQHandler(void) {
  OS_EnterInterrupt();
  // Add code here.
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       BSP_UART_Init()
*
*  Function description
*    Initialize the UART peripheral.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*    DataBits: Number of data bits to use.
*    Parity  : One of the following values:
*                - UART_PARITY_NONE
*                - UART_PARITY_ODD
*                - UART_PARITY_EVEN
*    StopBits: Number of stop bits to use.
*/
void BSP_UART_Init(unsigned Unit, U32 Baudrate, U8 DataBits, U8 Parity, U8 StopBits) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Baudrate);
  BSP_UART_USE_PARA(DataBits);
  BSP_UART_USE_PARA(Parity);
  BSP_UART_USE_PARA(StopBits);
  //
  // Setup UART interrupts.
  //
// NVIC_SetPriority(UART_RX_TX_IRQn, (1 << __NVIC_PRIO_BITS) - 2);
// NVIC_EnableIRQ(UART_RX_TX_IRQn);
// NVIC_SetPriority(UART_ERR_IRQn,   (1 << __NVIC_PRIO_BITS) - 2);
// NVIC_EnableIRQ(UART_ERR_IRQn);
}

/*********************************************************************
*
*       BSP_UART_DeInit()
*
*  Function description
*    De-initialize the UART peripheral.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*/
void BSP_UART_DeInit(unsigned Unit) {
  BSP_UART_USE_PARA(Unit);
}

/*********************************************************************
*
*       BSP_UART_SetBaudrate()
*
*  Function description
*    Configures the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
void BSP_UART_SetBaudrate(unsigned Unit, U32 Baudrate) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Baudrate);
}

/*********************************************************************
*
*       BSP_UART_SetReadCallback()
*
*  Function description
*    Sets the callback to execute upon an Rx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetReadCallback(unsigned Unit, void (*pf)(unsigned Unit, unsigned char Data)) {
  BSP_UART_USE_PARA(Unit);
}

/*********************************************************************
*
*       BSP_UART_SetWriteCallback()
*
*  Function description
*    Sets the callback to execute upon a Tx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetWriteCallback(unsigned Unit, int (*pf)(unsigned Unit)) {
  BSP_UART_USE_PARA(Unit);
}

/*********************************************************************
*
*       BSP_UART_Write1()
*
*  Function description
*    Sends one byte via UART.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    Data: First byte of data to send.
*
*  Additional information
*    Further bytes are typically sent from the Tx interrupt handler
*    by calling this function.
*/
void BSP_UART_Write1(unsigned Unit, U8 Data) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Data);
}

/*************************** End of file ****************************/
